@@ -11,7 +11,6 @@ from lightwood.api.types import (
     DataAnalysis,
 )
 from lightwood.api.predictor import PredictorInterface
-from lightwood.api.encode import encode
 from lightwood.api.high_level import (
     analyze_dataset,
     code_from_problem,
@@ -39,7 +38,6 @@ __all__ = [
     "ModelAnalysis",
     "DataAnalysis",
     "PredictorInterface",
-    "encode",
     "dtype",
     "predictor_from_state",
 ]
