@@ -1,11 +1,6 @@
-# TODO: We need a better way to specify trainable_encoders
 # TODO: lookup_encoder is awkward; similar to dtype, can we make a file with encoder_lookup? People may be interested
 # in seeing where these come from and it's not clear that you need to look here.
-# TODO: What does `target_class_distribution` and `positive_domain` do?
-# TODO: generate_json_ai is really large; can we abstract it into smaller functions to make it more readable?
 # TODO: add_implicit_values unit test ensures NO changes for a fully specified file.
-# TODO: Please fix spelling on parallel_preped_encoders
-
 from typing import Dict
 from lightwood.helpers.templating import call, inline_dict, align
 import black
@@ -19,10 +14,47 @@ from lightwood.api.types import (
     Output,
     ProblemDefinition,
 )
+import inspect
 
 
-trainable_encoders = ('PretrainedLangEncoder', 'CategoricalAutoEncoder', 'TimeSeriesEncoder', 'ArrayEncoder')
-ts_encoders = ('TimeSeriesEncoder', 'TsNumericEncoder')
+IMPORT_EXTERNAL_DIRS = """
+for import_dir in [os.path.expanduser('~/lightwood_modules'), '/etc/lightwood_modules']:
+    if os.path.exists(import_dir) and os.access(import_dir, os.R_OK):
+        for file_name in list(os.walk(import_dir))[0][2]:
+            if file_name[-3:] != '.py':
+                continue
+            mod_name = file_name[:-3]
+            loader = importlib.machinery.SourceFileLoader(mod_name,
+                                                          os.path.join(import_dir, file_name))
+            module = ModuleType(loader.name)
+            loader.exec_module(module)
+            sys.modules[mod_name] = module
+            exec(f'import {mod_name}')
+"""
+
+IMPORTS = """
+import lightwood
+from lightwood.analysis import *
+from lightwood.api import *
+from lightwood.data import *
+from lightwood.encoder import *
+from lightwood.ensemble import *
+from lightwood.helpers.device import *
+from lightwood.helpers.general import *
+from lightwood.helpers.log import *
+from lightwood.helpers.numeric import *
+from lightwood.helpers.parallelism import *
+from lightwood.helpers.seed import *
+from lightwood.helpers.text import *
+from lightwood.helpers.torch import *
+from lightwood.mixer import *
+import pandas as pd
+from typing import Dict, List
+import os
+from types import ModuleType
+import importlib.machinery
+import sys
+"""
 
 
 def lookup_encoder(
@@ -31,6 +63,7 @@ def lookup_encoder(
     is_target: bool,
     problem_defintion: ProblemDefinition,
     is_target_predicting_encoder: bool,
+    statistical_analysis: StatisticalAnalysis
 ):
     """
     Assign a default encoder for a given column based on its data type, and whether it is a target. Encoders intake raw (but cleaned) data and return an feature representation. This function assigns, per data type, what the featurizer should be. This function runs on each column within the dataset available for model building to assign how it should be featurized.
@@ -45,12 +78,15 @@ def lookup_encoder(
     :param problem_definition: The ``ProblemDefinition`` criteria; this populates specifics on how models and encoders may be trained.
     :param is_target_predicting_encoder:
     """ # noqa
+
     tss = problem_defintion.timeseries_settings
     encoder_lookup = {
         dtype.integer: 'Integer.NumericEncoder',
         dtype.float: 'Float.NumericEncoder',
         dtype.binary: 'Binary.BinaryEncoder',
-        dtype.categorical: 'Categorical.CategoricalAutoEncoder',
+        dtype.categorical: 'Categorical.CategoricalAutoEncoder'
+        if statistical_analysis is None or len(statistical_analysis.histograms[col_name]) > 100
+        else 'Categorical.OneHotEncoder',
         dtype.tags: 'Tags.MultiHotEncoder',
         dtype.date: 'Date.DatetimeEncoder',
         dtype.datetime: 'Datetime.DatetimeEncoder',
@@ -116,11 +152,10 @@ def lookup_encoder(
     if encoder_dict['module'] == "Rich_Text.PretrainedLangEncoder" and not is_target:
         encoder_dict['args']['output_type'] = "$dtype_dict[$target]"
 
-    for encoder_name in trainable_encoders:
-        if encoder_name == encoder_dict['module'].split(".")[1]:
-            encoder_dict['args'][
-                "stop_after"
-            ] = "$problem_definition.seconds_per_encoder"
+    if eval(encoder_dict['module'].split(".")[1]).is_trainable_encoder:
+        encoder_dict['args'][
+            "stop_after"
+        ] = "$problem_definition.seconds_per_encoder"
 
     if is_target_predicting_encoder:
         encoder_dict['args']['embed_mode'] = 'False'
@@ -140,7 +175,9 @@ def generate_json_ai(
     :param problem_definition: Specifies details of the model training/building procedure, as defined by ``ProblemDefinition``
 
     :returns: JSON-AI object with fully populated details of the ML pipeline
-    """ # noqa
+    """ # noqaexec
+    exec(IMPORTS, globals())
+    exec(IMPORT_EXTERNAL_DIRS, globals())
     target = problem_definition.target
     input_cols = []
     for col_name, col_dtype in type_information.dtypes.items():
@@ -153,6 +190,7 @@ def generate_json_ai(
 
     tss = problem_definition.timeseries_settings
     is_target_predicting_encoder = False
+    is_ts = problem_definition.timeseries_settings.is_timeseries
     # Single text column classification
     if (
         len(input_cols) == 1
@@ -224,6 +262,7 @@ def generate_json_ai(
             'module': 'BestOf',
             'args': {
                 'accuracy_functions': '$accuracy_functions',
+                'ts_analysis': 'self.ts_analysis' if is_ts else None
             }
         }
     )}
@@ -234,7 +273,7 @@ def generate_json_ai(
         list(outputs.values())[0].data_dtype = dtype.tsarray
 
     list(outputs.values())[0].encoder = lookup_encoder(
-        type_information.dtypes[target], target, True, problem_definition, False
+        type_information.dtypes[target], target, True, problem_definition, False, statistical_analysis
     )
 
     features: Dict[str, Feature] = {}
@@ -242,17 +281,16 @@ def generate_json_ai(
         col_dtype = type_information.dtypes[col_name]
         dependency = []
         encoder = lookup_encoder(
-            col_dtype, col_name, False, problem_definition, is_target_predicting_encoder
+            col_dtype, col_name, False, problem_definition, is_target_predicting_encoder, statistical_analysis
         )
 
-        for encoder_name in ts_encoders:
-            if tss.is_timeseries and encoder_name == encoder['module'].split(".")[1]:
-                if tss.group_by is not None:
-                    for group in tss.group_by:
-                        dependency.append(group)
+        if tss.is_timeseries and eval(encoder['module'].split(".")[1]).is_timeseries_encoder:
+            if tss.group_by is not None:
+                for group in tss.group_by:
+                    dependency.append(group)
 
-                if tss.use_previous_target:
-                    dependency.append(f"__mdb_ts_previous_{target}")
+            if tss.use_previous_target:
+                dependency.append(f"__mdb_ts_previous_{target}")
 
         if len(dependency) > 0:
             feature = Feature(encoder=encoder, dependency=dependency)
@@ -262,7 +300,7 @@ def generate_json_ai(
 
     # Decide on the accuracy functions to use
     output_dtype = list(outputs.values())[0].data_dtype
-    if output_dtype in [dtype.integer, dtype.float, dtype.date, dtype.datetime]:
+    if output_dtype in [dtype.integer, dtype.float, dtype.date, dtype.datetime, dtype.quantity]:
         accuracy_functions = ['r2_score']
     elif output_dtype in [dtype.categorical, dtype.tags, dtype.binary]:
         accuracy_functions = ['balanced_accuracy_score']
@@ -271,6 +309,11 @@ def generate_json_ai(
     else:
         raise Exception(f'Please specify a custom accuracy function for output type {output_dtype}')
 
+    # special dispatch for t+1 time series forecasters
+    if is_ts:
+        if list(outputs.values())[0].data_dtype in [dtype.integer, dtype.float]:
+            accuracy_functions = ['evaluate_array_accuracy']
+
     if problem_definition.time_aim is None and (
             problem_definition.seconds_per_mixer is None or problem_definition.seconds_per_encoder is None):
         problem_definition.time_aim = 1000 + np.log(
@@ -282,8 +325,8 @@ def generate_json_ai(
              for x in type_information.dtypes.values()]) * 200
 
     if problem_definition.time_aim is not None:
-        nr_trainable_encoders = len([x for x in features.values() if x.encoder['module'].split('.')[1]
-                                    in trainable_encoders])
+        nr_trainable_encoders = len([x for x in features.values() if
+                                    eval(x.encoder['module'].split('.')[1]).is_trainable_encoder])
         nr_mixers = len(list(outputs.values())[0].mixers)
         encoder_time_budget_pct = max(3.3 / 5, 1.5 + np.log(nr_trainable_encoders + 1) / 5)
 
@@ -302,7 +345,6 @@ def generate_json_ai(
         explainer=None,
         features=features,
         outputs=outputs,
-        imports=None,
         problem_definition=problem_definition,
         identifiers=type_information.identifiers,
         timeseries_transformer=None,
@@ -311,6 +353,59 @@ def generate_json_ai(
     )
 
 
+def merge_implicit_values(field, implicit_value):
+    exec(IMPORTS, globals())
+    exec(IMPORT_EXTERNAL_DIRS, globals())
+    module = eval(field['module'])
+    if inspect.isclass(module):
+        args = list(inspect.signature(module.__init__).parameters.keys())[1:]
+    else:
+        args = module.__code__.co_varnames
+
+    for arg in args:
+        if 'args' not in field:
+            field['args'] = implicit_value['args']
+        else:
+            if arg not in field['args']:
+                if arg in implicit_value['args']:
+                    field['args'][arg] = implicit_value['args'][arg]
+    return field
+
+
+def populate_implicit_field(json_ai: JsonAI, field_name: str, implicit_value: dict, is_timeseries: bool) -> None:
+    """
+    Populate the implicit field of the JsonAI, either by filling it in entirely if missing, or by introspecting the class or function and assigning default values to the args in it's signature that are in the implicit default but haven't been populated by the user
+
+    :params: json_ai: ``JsonAI`` object that describes the ML pipeline that may not have every detail fully specified.
+    :params: field_name: Name of the field the implicit field in ``JsonAI``
+    :params: implicit_value: The dictionary containing implicit values for the module and arg in the field
+    :params: is_timeseries: Whether or not this is a timeseries problem
+
+    :returns: nothing, this method mutates the respective field of the ``JsonAI`` object it receives
+    """ # noqa
+    # These imports might be slow, in which case the only <easy> solution is to line this code
+    field = json_ai.__getattribute__(field_name)
+    if field is None:
+        # This if is to only populated timeseries-specific implicit fields for implicit problems
+        if is_timeseries or field_name not in ('timeseries_analyzer', 'timeseries_transformer'):
+            field = implicit_value
+
+    # If the user specified one or more subfields in a field that's a list
+    # Populate them with implicit arguments form the implicit values from that subfield
+    elif isinstance(field, list) and isinstance(implicit_value, list):
+        for i in range(len(field)):
+            sub_field_implicit = [x for x in implicit_value if x['module'] == field[i]['module']]
+            if len(sub_field_implicit) == 1:
+                field[i] = merge_implicit_values(field[i], sub_field_implicit[0])
+        for sub_field_implicit in implicit_value:
+            if len([x for x in field if x['module'] == sub_field_implicit['module']]) == 0:
+                field.append(sub_field_implicit)
+    # If the user specified the field, add implicit arguments which we didn't specify
+    else:
+        field = merge_implicit_values(field, implicit_value)
+    json_ai.__setattr__(field_name, field)
+
+
 def add_implicit_values(json_ai: JsonAI) -> JsonAI:
     """
     To enable brevity in writing, auto-generate the "unspecified/missing" details required in the ML pipeline.
@@ -322,36 +417,6 @@ def add_implicit_values(json_ai: JsonAI) -> JsonAI:
     problem_definition = json_ai.problem_definition
     tss = problem_definition.timeseries_settings
 
-    imports = [
-        'from lightwood.mixer import Neural', 'from lightwood.mixer import LightGBM',
-        'from lightwood.mixer import LightGBMArray', 'from lightwood.mixer import SkTime',
-        'from lightwood.mixer import Unit', 'from lightwood.mixer import Regression',
-        'from lightwood.ensemble import BestOf', 'from lightwood.data import cleaner',
-        'from lightwood.data import transform_timeseries, timeseries_analyzer', 'from lightwood.data import splitter',
-        'from lightwood.analysis import model_analyzer, explain',
-        'from sklearn.metrics import r2_score, balanced_accuracy_score, accuracy_score', 'import pandas as pd',
-        'from lightwood.helpers.seed import seed', 'from lightwood.helpers.log import log', 'import lightwood',
-        'from lightwood.api import *', 'from lightwood.mixer import BaseMixer',
-        'from lightwood.encoder import BaseEncoder, __ts_encoders__',
-        'from lightwood.encoder import Array, Binary, Categorical, Date, Datetime, TimeSeries, Float, Image, Integer, Quantity, Rich_Text, Short_Text, Tags', # noqa
-        'from lightwood.ensemble import BaseEnsemble', 'from typing import Dict, List',
-        'from lightwood.helpers.parallelism import mut_method_call',
-        'from lightwood.data.encoded_ds import ConcatedEncodedDs', 'from lightwood import ProblemDefinition']
-
-    if json_ai.imports is None:
-        json_ai.imports = imports
-    else:
-        json_ai.imports.extend(imports)
-
-    for feature in [list(json_ai.outputs.values())[0], *json_ai.features.values()]:
-        encoder_import = feature.encoder['module']
-        if "." in encoder_import:
-            continue
-        imports.append(f"from lightwood.encoder import {encoder_import}")
-
-    if tss.use_previous_target:
-        imports.append('from lightwood.encoder import ArrayEncoder')
-
     # Add implicit arguments
     # @TODO: Consider removing once we have a proper editor in studio
     mixers = json_ai.outputs[json_ai.problem_definition.target].mixers
@@ -389,7 +454,7 @@ def add_implicit_values(json_ai: JsonAI) -> JsonAI:
 
     ensemble = json_ai.outputs[json_ai.problem_definition.target].ensemble
     ensemble['args']['target'] = ensemble['args'].get('target', '$target')
-    ensemble['args']['data'] = ensemble['args'].get('data', 'test_data')
+    ensemble['args']['data'] = ensemble['args'].get('data', 'encoded_test_data')
     ensemble['args']['mixers'] = ensemble['args'].get('mixers', '$mixers')
 
     for name in json_ai.features:
@@ -400,95 +465,105 @@ def add_implicit_values(json_ai: JsonAI) -> JsonAI:
                 json_ai.features[name].encoder['module'].split(".")[0].lower()
             )
 
-    # Add implicit phases
-    # @TODO: Consider removing once we have a proper editor in studio
-    if json_ai.cleaner is None:
-        json_ai.cleaner = {
-            "module": "cleaner",
-            "args": {
-                "pct_invalid": "$problem_definition.pct_invalid",
-                "ignore_features": "$problem_definition.ignore_features",
-                "identifiers": "$identifiers",
-                "data": "data",
-                "dtype_dict": "$dtype_dict",
-                "target": "$target",
-                "mode": "$mode",
-                "timeseries_settings": "$problem_definition.timeseries_settings",
-                "anomaly_detection": "$problem_definition.anomaly_detection",
-            },
+    # Add "hidden" fields
+    hidden_fields = [('cleaner', {
+        "module": "cleaner",
+        "args": {
+            "pct_invalid": "$problem_definition.pct_invalid",
+            "identifiers": "$identifiers",
+            "data": "data",
+            "dtype_dict": "$dtype_dict",
+            "target": "$target",
+            "mode": "$mode",
+            "timeseries_settings": "$problem_definition.timeseries_settings",
+            "anomaly_detection": "$problem_definition.anomaly_detection",
+        },
+    }), ('splitter', {
+        'module': 'splitter',
+        'args': {
+            'tss': '$problem_definition.timeseries_settings',
+            'data': 'data',
+            'seed': 1,
+            'target': '$target',
+            'dtype_dict': '$dtype_dict',
+            'pct_train': 80,
+            'pct_dev': 10,
+            'pct_test': 10
         }
+    }), ('analyzer', {
+         "module": "model_analyzer",
+         "args": {
+             "stats_info": "$statistical_analysis",
+             "ts_cfg": "$problem_definition.timeseries_settings",
+             "accuracy_functions": "$accuracy_functions",
+             "predictor": "$ensemble",
+             "data": "encoded_test_data",
+             "train_data": "encoded_train_data",
+             "target": "$target",
+             "dtype_dict": "$dtype_dict",
+             "analysis_blocks": "$analysis_blocks"
+         },
+         }), ('explainer', {
+             "module": "explain",
+             "args": {
+                 "timeseries_settings": "$problem_definition.timeseries_settings",
+                 "positive_domain": "$statistical_analysis.positive_domain",
+                 "fixed_confidence": "$problem_definition.fixed_confidence",
+                 "anomaly_detection": "$problem_definition.anomaly_detection",
+                 "anomaly_error_rate": "$problem_definition.anomaly_error_rate",
+                 "anomaly_cooldown": "$problem_definition.anomaly_cooldown",
+                 "data": "data",
+                 "encoded_data": "encoded_data",
+                 "predictions": "df",
+                 "analysis": "$runtime_analyzer",
+                 "ts_analysis": "$ts_analysis" if tss.is_timeseries else None,
+                 "target_name": "$target",
+                 "target_dtype": "$dtype_dict[self.target]",
+                 "explainer_blocks": "$analysis_blocks"
+             },
+         }), ('analysis_blocks', [
+             {
+                 'module': 'ICP',
+                 'args': {
+                     "fixed_significance": None,
+                     "confidence_normalizer": False,
+                     "positive_domain": "$statistical_analysis.positive_domain",
 
-    if json_ai.splitter is None:
-        json_ai.splitter = {
-            'module': 'splitter',
-            'args': {
-                'tss': '$problem_definition.timeseries_settings',
-                'data': 'data',
-                'k': 'nsubsets'
-            }
-        }
-    if json_ai.analyzer is None:
-        json_ai.analyzer = {
-            "module": "model_analyzer",
-            "args": {
-                "stats_info": "$statistical_analysis",
-                "ts_cfg": "$problem_definition.timeseries_settings",
-                "accuracy_functions": "$accuracy_functions",
-                "predictor": "$ensemble",
-                "data": "test_data",
-                "train_data": "train_data",
-                "target": "$target",
-                "disable_column_importance": "False",
-                "dtype_dict": "$dtype_dict",
-                "fixed_significance": None,
-                "confidence_normalizer": False,
-                "positive_domain": "$statistical_analysis.positive_domain",
-            },
-        }
+                 },
+             },
+             {
+                 'module': 'AccStats',
+                 'args': {
+                     'deps': ['ICP']
+                 },
+             },
+             {
+                 'module': 'GlobalFeatureImportance',
+                 'args': {
+                     "disable_column_importance": "False",
+                 },
+             },
+         ]), ('timeseries_transformer', {
+             "module": "transform_timeseries",
+             "args": {
+                 "timeseries_settings": "$problem_definition.timeseries_settings",
+                 "data": "data",
+                 "dtype_dict": "$dtype_dict",
+                 "target": "$target",
+                 "mode": "$mode",
+             },
+         }), ('timeseries_analyzer', {
+             "module": "timeseries_analyzer",
+             "args": {
+                 "timeseries_settings": "$problem_definition.timeseries_settings",
+                 "data": "data",
+                 "dtype_dict": "$dtype_dict",
+                 "target": "$target",
+             },
+         })]
 
-    if json_ai.explainer is None:
-        json_ai.explainer = {
-            "module": "explain",
-            "args": {
-                "timeseries_settings": "$problem_definition.timeseries_settings",
-                "positive_domain": "$statistical_analysis.positive_domain",
-                "fixed_confidence": "$problem_definition.fixed_confidence",
-                "anomaly_detection": "$problem_definition.anomaly_detection",
-                "anomaly_error_rate": "$problem_definition.anomaly_error_rate",
-                "anomaly_cooldown": "$problem_definition.anomaly_cooldown",
-                "data": "data",
-                "encoded_data": "encoded_data",
-                "predictions": "df",
-                "analysis": "$runtime_analyzer",
-                "ts_analysis": "$ts_analysis" if tss.is_timeseries else None,
-                "target_name": "$target",
-                "target_dtype": "$dtype_dict[self.target]",
-            },
-        }
-
-    if tss.is_timeseries:
-        if json_ai.timeseries_transformer is None:
-            json_ai.timeseries_transformer = {
-                "module": "transform_timeseries",
-                "args": {
-                    "timeseries_settings": "$problem_definition.timeseries_settings",
-                    "data": "data",
-                    "dtype_dict": "$dtype_dict",
-                    "target": "$target",
-                    "mode": "$mode",
-                },
-            }
-
-        if json_ai.timeseries_analyzer is None:
-            json_ai.timeseries_analyzer = {
-                "module": "timeseries_analyzer",
-                "args": {
-                    "timeseries_settings": "$problem_definition.timeseries_settings",
-                    "data": "data",
-                    "dtype_dict": "$dtype_dict",
-                    "target": "$target",
-                },
-            }
+    for field_name, implicit_value in hidden_fields:
+        populate_implicit_field(json_ai, field_name, implicit_value, tss.is_timeseries)
 
     return json_ai
 
@@ -511,10 +586,9 @@ def code_from_json_ai(json_ai: JsonAI) -> str:
     }
 
     for col_name, feature in json_ai.features.items():
-        if col_name not in json_ai.problem_definition.ignore_features:
-            encoder_dict[col_name] = call(feature.encoder)
-            dependency_dict[col_name] = feature.dependency
-            dtype_dict[col_name] = f"""'{feature.data_dtype}'"""
+        encoder_dict[col_name] = call(feature.encoder)
+        dependency_dict[col_name] = feature.dependency
+        dtype_dict[col_name] = f"""'{feature.data_dtype}'"""
 
     # @TODO: Move into json-ai creation function (I think? Maybe? Let's discuss)
     tss = json_ai.problem_definition.timeseries_settings
@@ -526,15 +600,15 @@ def code_from_json_ai(json_ai: JsonAI) -> str:
                                                      False,
                                                      json_ai.problem_definition,
                                                      False,
+                                                     None
                                                      ))
         dependency_dict[col_name] = []
         dtype_dict[col_name] = f"""'{list(json_ai.outputs.values())[0].data_dtype}'"""
         json_ai.features[col_name] = Feature(encoder=encoder_dict[col_name])
 
-    ignored_cols = json_ai.problem_definition.ignore_features
     input_cols = [x.replace("'", "\\'").replace('"', '\\"')
                   for x in json_ai.features]
-    input_cols = ','.join([f"""'{name}'""" for name in input_cols if name not in ignored_cols])
+    input_cols = ','.join([f"""'{name}'""" for name in input_cols])
 
     ts_transform_code = ""
     ts_analyze_code = ""
@@ -550,7 +624,7 @@ self.ts_analysis = {call(json_ai.timeseries_analyzer)}
 
     if json_ai.timeseries_analyzer is not None:
         ts_encoder_code = """
-if type(encoder) in __ts_encoders__:
+if encoder.is_timeseries_encoder:
     kwargs['ts_analysis'] = self.ts_analysis
 """
 
@@ -565,7 +639,6 @@ if encoder.is_target:
 
     dataprep_body = f"""
 # The type of each column
-self.problem_definition = ProblemDefinition.from_dict({json_ai.problem_definition.to_dict()})
 self.accuracy_functions = {json_ai.accuracy_functions}
 self.identifiers = {json_ai.identifiers}
 self.dtype_dict = {inline_dict(dtype_dict)}
@@ -579,35 +652,38 @@ self.dependencies = {inline_dict(dependency_dict)}
 #
 self.input_cols = [{input_cols}]
 
+self.analysis_blocks = [{', '.join([call(block) for block in json_ai.analysis_blocks])}]
+
 log.info('Cleaning the data')
 data = {call(json_ai.cleaner)}
 
 {ts_transform_code}
 {ts_analyze_code}
 
-nsubsets = {json_ai.problem_definition.nsubsets}
-log.info(f'Splitting the data into {{nsubsets}} subsets')
-subsets = {call(json_ai.splitter)}
+data = {call(json_ai.splitter)}
 
 log.info('Preparing the encoders')
 
-encoder_preping_dict = {{}}
-enc_preping_data = pd.concat(subsets[0:nsubsets-1])
+encoder_prepping_dict = {{}}
+concatenated_train_dev = pd.concat([data['train'], data['dev']])
 for col_name, encoder in self.encoders.items():
-    if not encoder.is_nn_encoder:
-        encoder_preping_dict[col_name] = [encoder, enc_preping_data[col_name], 'prepare']
-        log.info(f'Encoder preping dict length of: {{len(encoder_preping_dict)}}')
+    if not encoder.is_trainable_encoder:
+        encoder_prepping_dict[col_name] = [encoder, concatenated_train_dev[col_name], 'prepare']
+        log.info(f'Encoder prepping dict length of: {{len(encoder_prepping_dict)}}')
 
-parallel_preped_encoders = mut_method_call(encoder_preping_dict)
-for col_name, encoder in parallel_preped_encoders.items():
+parallel_prepped_encoders = mut_method_call(encoder_prepping_dict)
+for col_name, encoder in parallel_prepped_encoders.items():
     self.encoders[col_name] = encoder
 
-if self.target not in parallel_preped_encoders:
-    self.encoders[self.target].prepare(enc_preping_data[self.target])
+if self.target not in parallel_prepped_encoders:
+    if self.encoders[self.target].is_trainable_encoder:
+        self.encoders[self.target].prepare(data['train'][self.target], data['dev'][self.target])
+    else:
+        self.encoders[self.target].prepare(pd.concat([data['train'], data['dev']])[self.target])
 
 for col_name, encoder in self.encoders.items():
-    if encoder.is_nn_encoder:
-        priming_data = pd.concat(subsets[0:nsubsets-1])
+    if encoder.is_trainable_encoder:
+        priming_data = pd.concat([data['train'], data['dev']])
         kwargs = {{}}
         if self.dependencies[col_name]:
             kwargs['dependency_data'] = {{}}
@@ -620,9 +696,9 @@ for col_name, encoder in self.encoders.items():
 
         # This assumes target  encoders are also prepared in parallel, might not be true
         if hasattr(encoder, 'uses_target'):
-            kwargs['encoded_target_values'] = parallel_preped_encoders[self.target].encode(priming_data[self.target])
+            kwargs['encoded_target_values'] = parallel_prepped_encoders[self.target].encode(priming_data[self.target])
 
-        encoder.prepare(priming_data[col_name], **kwargs)
+        encoder.prepare(data['train'][col_name], data['dev'][col_name], **kwargs)
 
     {align(ts_target_code, 1)}
 """
@@ -631,16 +707,16 @@ for col_name, encoder in self.encoders.items():
     learn_body = f"""
 log.info('Featurizing the data')
 
-encoded_ds_arr = lightwood.encode(self.encoders, subsets, self.target)
-train_data = encoded_ds_arr[0:int(nsubsets*0.9)]
-test_data = encoded_ds_arr[int(nsubsets*0.9):]
+encoded_train_data = EncodedDs(self.encoders, data['train'], self.target)
+encoded_dev_data = EncodedDs(self.encoders, data['dev'], self.target)
+encoded_test_data = EncodedDs(self.encoders, data['test'], self.target)
 
 log.info('Training the mixers')
 self.mixers = [{', '.join([call(x) for x in list(json_ai.outputs.values())[0].mixers])}]
 trained_mixers = []
 for mixer in self.mixers:
     try:
-        mixer.fit(train_data)
+        mixer.fit(encoded_train_data, encoded_dev_data)
         trained_mixers.append(mixer)
     except Exception as e:
         log.warning(f'Exception: {{e}} when training mixer: {{mixer}}')
@@ -661,41 +737,44 @@ self.model_analysis, self.runtime_analyzer = {call(json_ai.analyzer)}
 # important to train with.
 for mixer in self.mixers:
     if {json_ai.problem_definition.fit_on_validation}:
-        mixer.partial_fit(test_data, train_data)
+        mixer.partial_fit(encoded_test_data, ConcatedEncodedDs([encoded_train_data, encoded_dev_data]))
 """
     learn_body = align(learn_body, 2)
 
     predict_common_body = f"""
+log.info(f'Dropping features: {{self.problem_definition.ignore_features}}')
+data = data.drop(columns=self.problem_definition.ignore_features)
+for col in self.input_cols:
+    if col not in data.columns:
+        data[col] = [None] * len(data)
 self.mode = 'predict'
 log.info('Cleaning the data')
 data = {call(json_ai.cleaner)}
 
 {ts_transform_code}
 
-encoded_ds = lightwood.encode(self.encoders, data, self.target)[0]
+encoded_ds = EncodedDs(self.encoders, data, self.target)
 encoded_data = encoded_ds.get_encoded_data(include_target=False)
 """
     predict_common_body = align(predict_common_body, 2)
 
     predict_body = f"""
 df = self.ensemble(encoded_ds)
-insights = {call(json_ai.explainer)}
+insights, global_insights = {call(json_ai.explainer)}
 return insights
 """
     predict_body = align(predict_body, 2)
 
     predict_proba_body = f"""
 df = self.ensemble(encoded_ds, predict_proba=True)
-insights = {call(json_ai.explainer)}
+insights, global_insights = {call(json_ai.explainer)}
 return insights
 """
     predict_proba_body = align(predict_proba_body, 2)
 
-    imports = "\n".join(json_ai.imports)
     predictor_code = f"""
-{imports}
-from lightwood.api import PredictorInterface
-
+{IMPORTS}
+{IMPORT_EXTERNAL_DIRS}
 
 class Predictor(PredictorInterface):
     target: str
@@ -710,6 +789,9 @@ class Predictor(PredictorInterface):
         self.mode = 'innactive'
 
     def learn(self, data: pd.DataFrame) -> None:
+        self.problem_definition = ProblemDefinition.from_dict({json_ai.problem_definition.to_dict()})
+        log.info(f'Dropping features: {{self.problem_definition.ignore_features}}')
+        data = data.drop(columns=self.problem_definition.ignore_features)
 {dataprep_body}
 {learn_body}
 
