@@ -10,7 +10,6 @@
 # TODO: Problem definition missing a few terms
 # TODO: Model Analysis
 # TODO: Analyzer
-
 from typing import Dict, List, Optional, Union
 from dataclasses import dataclass
 from lightwood.helpers.log import log
@@ -106,8 +105,8 @@ class Output:
     :param data_dtype: The type of information within the target column (ex.: numerical, categorical, etc.).
     :param encoder: the methodology for encoding the target feature (a Lightwood Encoder). There can only be one \
     encoder for the output target.
-    :param models: The list of ML algorithms that are trained for the target distribution.
-    :param ensemble: For a panel of ML algorithms, the approach of selecting the best model, and the metrics used in \
+    :param mixers: The list of ML algorithms that are trained for the target distribution.
+    :param ensemble: For a panel of ML algorithms, the approach of selecting the best mixer, and the metrics used in \
     that evaluation.
     """
 
@@ -216,6 +215,7 @@ class TimeseriesSettings:
     historical_columns: List[str] = None
     target_type: str = (
         ""  # @TODO: is the current setter (outside of initialization) a sane option?
+        # @TODO: George: No, I don't think it is, we need to pass this some other way
     )
     allow_incomplete_history: bool = False
 
@@ -318,7 +318,6 @@ class ProblemDefinition:
     """
 
     target: str
-    nsubsets: int
     pct_invalid: float
     unbias_target: bool
     seconds_per_mixer: Union[int, None]
@@ -347,8 +346,7 @@ class ProblemDefinition:
         :returns: A populated ``ProblemDefinition`` object.
         """
         target = obj['target']
-        nsubsets = obj.get('nsubsets', 30)
-        pct_invalid = obj.get('pct_invalid', 1)
+        pct_invalid = obj.get('pct_invalid', 2)
         unbias_target = obj.get('unbias_target', True)
         seconds_per_mixer = obj.get('seconds_per_mixer', None)
         seconds_per_encoder = obj.get('seconds_per_encoder', None)
@@ -366,7 +364,6 @@ class ProblemDefinition:
         seed_nr = obj.get('seed_nr', 420)
         problem_definition = ProblemDefinition(
             target=target,
-            nsubsets=nsubsets,
             pct_invalid=pct_invalid,
             unbias_target=unbias_target,
             seconds_per_mixer=seconds_per_mixer,
@@ -430,10 +427,10 @@ class JsonAI:
     :param splitter: The Splitter object is the method in which the input data is split into training/validation/testing data.
     :param analyzer: The Analyzer object is used to evaluate how well a model performed on the predictive task.
     :param explainer: The Explainer object deploys explainability tools of interest on a model to indicate how well a model generalizes its predictions.
-    :param imports: A list of custom packages, indicated through a str import statement, that a user can call.
+    :param analysis_blocks: The blocks that get used in both analysis and inference inside the analyzer and explainer blocks.
     :param timeseries_transformer:
     :param timeseries_analyzer:
-    :param accuracy_functions: A list of performance metrics used to evaluate the best models.
+    :param accuracy_functions: A list of performance metrics used to evaluate the best mixers.
     """ # noqa
 
     features: Dict[str, Feature]
@@ -444,11 +441,10 @@ class JsonAI:
     splitter: Optional[object] = None
     analyzer: Optional[object] = None
     explainer: Optional[object] = None
-    imports: Optional[List[str]] = None
+    analysis_blocks: Optional[List[object]] = None
     timeseries_transformer: Optional[object] = None
     timeseries_analyzer: Optional[object] = None
     accuracy_functions: Optional[List[str]] = None
-    phases: Optional[Dict[str, object]] = None
 
     @staticmethod
     def from_dict(obj: Dict):
@@ -463,11 +459,10 @@ class JsonAI:
         splitter = obj.get("splitter", None)
         analyzer = obj.get("analyzer", None)
         explainer = obj.get("explainer", None)
-        imports = obj.get("imports", None)
+        analysis_blocks = obj.get("analysis_blocks", None)
         timeseries_transformer = obj.get("timeseries_transformer", None)
         timeseries_analyzer = obj.get("timeseries_analyzer", None)
         accuracy_functions = obj.get("accuracy_functions", None)
-        phases = obj.get("phases", None)
 
         json_ai = JsonAI(
             features=features,
@@ -478,11 +473,10 @@ class JsonAI:
             splitter=splitter,
             analyzer=analyzer,
             explainer=explainer,
-            imports=imports,
+            analysis_blocks=analysis_blocks,
             timeseries_transformer=timeseries_transformer,
             timeseries_analyzer=timeseries_analyzer,
             accuracy_functions=accuracy_functions,
-            phases=phases,
         )
 
         return json_ai
