@@ -10,16 +10,16 @@ from lightwood.data.cleaner import cleaner
 from lightwood.helpers.log import log
 from lightwood.api.dtype import dtype
 from scipy.stats import entropy
-from lightwood.data.cleaner import _clean_float_or_none
+from lightwood.data.cleaner import _clean_float
 
 
 def get_datetime_histogram(data: pd.Series, bins: int) -> Dict[str, list]:
     """Generates the histogram for date and datetime types
     """
     if isinstance(data[0], float) or isinstance(data[0], int):
-        data = [_clean_float_or_none(x) for x in data]
+        data = [_clean_float(x) for x in data]
     else:
-        data = [_clean_float_or_none(parse_dt(str(x)).timestamp()) for x in data]
+        data = [_clean_float(parse_dt(str(x)).timestamp()) for x in data]
 
     Y, X = np.histogram(data, bins=min(bins, len(set(data))),
                         range=(min(data), max(data)), density=False)
@@ -44,7 +44,7 @@ def get_numeric_histogram(data: pd.Series, data_dtype: dtype, bins: int) -> Dict
             new_data.extend(arr)
         data = new_data
 
-    data = [_clean_float_or_none(x) for x in data]
+    data = [_clean_float(x) for x in data]
 
     Y, X = np.histogram(data, bins=min(bins, len(set(data))),
                         range=(min(data), max(data)), density=False)
@@ -80,7 +80,7 @@ def statistical_analysis(data: pd.DataFrame,
                          seed_nr: int = 420) -> StatisticalAnalysis:
     seed(seed_nr)
     log.info('Starting statistical analysis')
-    df = cleaner(data, dtypes, problem_definition.pct_invalid, problem_definition.ignore_features,
+    df = cleaner(data, dtypes, problem_definition.pct_invalid,
                  identifiers, problem_definition.target, 'train', problem_definition.timeseries_settings,
                  problem_definition.anomaly_detection)
 
@@ -91,7 +91,7 @@ def statistical_analysis(data: pd.DataFrame,
     target = problem_definition.target
     positive_domain = False
     # get train std, used in analysis
-    if dtypes[target] in [dtype.float, dtype.integer, dtype.tsarray]:
+    if dtypes[target] in [dtype.float, dtype.integer, dtype.tsarray, dtype.quantity]:
         df_std = df[target].astype(float).std()
         if min(df[target]) >= 0:
             positive_domain = True
@@ -122,7 +122,7 @@ def statistical_analysis(data: pd.DataFrame,
                 'y': list(hist.values())
             }
             buckets[col] = histograms[col]['x']
-        elif dtypes[col] in (dtype.integer, dtype.float, dtype.array, dtype.tsarray):
+        elif dtypes[col] in (dtype.integer, dtype.float, dtype.array, dtype.tsarray, dtype.quantity):
             histograms[col] = get_numeric_histogram(filter_nan_and_none(df[col]), dtypes[col], 50)
             buckets[col] = histograms[col]['x']
         elif dtypes[col] in (dtype.date, dtype.datetime):
@@ -155,7 +155,8 @@ def statistical_analysis(data: pd.DataFrame,
         if dtypes[col] in (dtype.rich_text, dtype.short_text):
             words_per_sentence = []
             for item in df[col]:
-                words_per_sentence.append(len(item.split(' ')))
+                if item is not None:
+                    words_per_sentence.append(len(item.split(' ')))
             avg_words_per_sentence[col] = int(np.mean(words_per_sentence))
         else:
             avg_words_per_sentence[col] = None
