@@ -7,11 +7,13 @@ class BaseEncoder:
     is_target: bool
     prepared: bool
 
+    is_timeseries_encoder: bool = False
+    is_trainable_encoder: bool = False
+
     def __init__(self, is_target=False) -> None:
         self.is_target = is_target
         self._prepared = False
         self.uses_subsets = False
-        self.is_nn_encoder = False
         self.dependencies = []
         self.output_size = None
 
@@ -25,7 +27,7 @@ class BaseEncoder:
     def decode(self, encoded_data) -> List[object]:
         raise NotImplementedError
 
-    # Should work for all troch-based encoders, but custom behavior may have to be implemented for very weird models
+    # Should work for all torch-based encoders, but custom behavior may have to be implemented for weird models
     def to(self, device, available_devices):
         # Find all nn.Module type objects and convert them
         # @TODO: Make this work recursively
