@@ -1,7 +1,7 @@
 from lightwood.encoder.numeric.numeric import NumericEncoder
 from lightwood.encoder.numeric.ts_numeric import TsNumericEncoder
 from lightwood.encoder.time_series.rnn import TimeSeriesEncoder
-from lightwood.encoder.time_series.plain import TimeSeriesPlainEncoder
+from lightwood.encoder.array.array import ArrayEncoder
 
 
-__all__ = ['NumericEncoder', 'TsNumericEncoder', 'TimeSeriesEncoder', 'TimeSeriesPlainEncoder']
+__all__ = ['NumericEncoder', 'TsNumericEncoder', 'TimeSeriesEncoder', 'ArrayEncoder']
