@@ -1,4 +1,3 @@
-from typing import List
 import pandas as pd
 from lightwood.data.encoded_ds import EncodedDs
 
@@ -11,11 +10,11 @@ class BaseMixer:
         self.stop_after = stop_after
         self.supports_proba = None
 
-    def fit(self, data: List[EncodedDs]) -> None:
+    def fit(self, train_data: EncodedDs, dev_data: EncodedDs) -> None:
         raise NotImplementedError()
 
     def __call__(self, ds: EncodedDs, predict_proba: bool = False) -> pd.DataFrame:
         raise NotImplementedError()
 
-    def partial_fit(self, train_data: List[EncodedDs], test_data: List[EncodedDs]) -> None:
+    def partial_fit(self, train_data: EncodedDs, dev_data: EncodedDs) -> None:
         pass
